@@ -254,8 +254,10 @@ void KConfigHeaderGenerator::createSignals()
             stream() << whitespace() << "*/\n";
         }
         stream() << whitespace() << "void " << signal.name << "(";
-        QList<Param>::ConstIterator it, itEnd = signal.arguments.constEnd();
-        for (it = signal.arguments.constBegin(); it != itEnd;) {
+
+        auto it = signal.arguments.cbegin();
+        const auto itEnd = signal.arguments.cend();
+        while (it != itEnd) {
             Param argument = *it;
             QString type = param(argument.type);
             if (cfg().useEnumTypes && argument.type == QLatin1String("Enum")) {
@@ -345,7 +347,7 @@ void KConfigHeaderGenerator::createConstructor()
 
 void KConfigHeaderGenerator::createDestructor()
 {
-    stream() << whitespace() << "~" << cfg().className << "();\n\n";
+    stream() << whitespace() << "~" << cfg().className << "() override;\n\n";
 }
 
 void KConfigHeaderGenerator::createForwardDeclarations()
